from pathlib import Path
from databricksbundle.pipeline.function.ServiceResolver import ServiceResolver
from databricksbundle.pipeline.function.functionInspector import inspectFunction

class ServicesResolver:

    def __init__(
        self,
        serviceResolver: ServiceResolver,
    ):
        self.__serviceResolver = serviceResolver

    def resolve(self, fun: callable, startIndex: int, pipelinePath: Path):
        functionArguments = inspectFunction(fun)

        def resolve(inspectedArgument):
            return self.__serviceResolver.resolve(inspectedArgument.dtype, pipelinePath)

        return tuple(map(resolve, functionArguments[startIndex:]))



# pylint: disable = invalid-name
import sys
from pathlib import Path
from typing import Tuple
from databricksbundle.detector import isDatabricks
from databricksbundle.notebook.helpers import isNotebookEnvironment
from databricksbundle.pipeline.decorator.containerLoader import containerInitEnvVarDefined
from databricksbundle.pipeline.function.ServicesResolver import ServicesResolver
from databricksbundle.pipeline.decorator.argsChecker import checkArgs
from databricksbundle.pipeline.decorator.executor.dataFrameLoader import loadDataFrame
from databricksbundle.pipeline.decorator.executor.transformation import transform
from databricksbundle.pipeline.decorator.executor.dataFrameSaver import saveDataFrame

if containerInitEnvVarDefined():
    from databricksbundle.container.envVarContainerLoader import container
else:
    from databricksbundle.container.pyprojectContainerLoader import container

def _getPipelinePath():
    if isDatabricks() and not isNotebookEnvironment():
        return Path(sys.argv[1])  # calling from Databricks job
    else:
        return Path(sys.argv[0])

def _resolveServices(fun, index: int):
    return container.get(ServicesResolver).resolve(fun, index, _getPipelinePath())  # pylint: disable = no-member

class pipelineFunction:

    def __init__(self, *args, **kwargs): # pylint: disable = unused-argument
        checkArgs(args, self.__class__.__name__)

    def __call__(self, fun, *args, **kwargs):
        services = _resolveServices(fun, 0)
        fun(*services)

        return fun

class dataFrameLoader:

    def __init__(self, *args, **kwargs): # pylint: disable = unused-argument
        checkArgs(args, self.__class__.__name__)

    def __call__(self, fun, *args, **kwargs):
        services = _resolveServices(fun, 0)
        loadDataFrame(fun, services)

        return fun

class transformation:

    def __init__(self, *args, **kwargs): # pylint: disable = unused-argument
        self._sources = args # type: Tuple[callable]

    def __call__(self, fun, *args, **kwargs):
        services = _resolveServices(fun, len(self._sources))
        transform(fun, self._sources, services)

        return fun

class dataFrameSaver:

    def __init__(self, *args):
        self._sources = args # type: Tuple[callable]

    def __call__(self, fun, *args, **kwargs):
        services = _resolveServices(fun, 1)
        saveDataFrame(fun, self._sources, services)

        return fun



from pathlib import Path
from typing import Dict
from injecta.container.ContainerInterface import ContainerInterface
from injecta.dtype.DType import DType
from injecta.dtype.classLoader import loadClass
from databricksbundle.pipeline.function.service.ServiceResolverInterface import ServiceResolverInterface

class ServiceResolver:

    def __init__(
        self,
        serviceResolvers: Dict[str, str],
        container: ContainerInterface
    ):
        self.__serviceResolversMapping = serviceResolvers
        self.__container = container

    def resolve(self, dtype: DType, pipelinePath: Path):
        argumentType = str(dtype)

        if argumentType in self.__serviceResolversMapping:
            if self.__serviceResolversMapping[argumentType][0:1] != '@':
                raise Exception('Service name must start with @')

            serviceResolverName = self.__serviceResolversMapping[argumentType][1:]
            serviceResolver = self.__container.get(serviceResolverName) # type: ServiceResolverInterface

            return serviceResolver.resolve(pipelinePath)

        class_ = loadClass(dtype.moduleName, dtype.className) # pylint: disable = invalid-name

        return self.__container.get(class_)



from typing import List, Tuple

def saveDataFrame(fun: callable, sources: Tuple[callable], services: List[object]):
    g = fun.__globals__

    def transformSource(source: callable):
        return g[source.__name__ + '_df']

    dataframesToUse = tuple(map(transformSource, sources))

    fun(*(dataframesToUse + services))



# pylint: disable = unused-import
from databricksbundle.detector import isDatabricks
from databricksbundle.notebook.helpers import isNotebookEnvironment

if isDatabricks() and isNotebookEnvironment():
    from databricksbundle.pipeline.decorator.environment.databricksNotebook import pipelineFunction, dataFrameLoader, transformation, dataFrameSaver
else:
    from databricksbundle.pipeline.decorator.environment.pyscript import pipelineFunction, dataFrameLoader, transformation, dataFrameSaver



# pylint: disable = invalid-name
from pathlib import Path
from typing import Tuple
from databricksbundle.display import display
from databricksbundle.pipeline.function.ServicesResolver import ServicesResolver
from databricksbundle.pipeline.decorator.argsChecker import checkArgs
from databricksbundle.notebook.helpers import getNotebookPath
from databricksbundle.pipeline.decorator.executor.dataFrameLoader import loadDataFrame
from databricksbundle.pipeline.decorator.executor.transformation import transform
from databricksbundle.pipeline.decorator.executor.dataFrameSaver import saveDataFrame
from databricksbundle.container.envVarContainerLoader import container

def _resolveServices(fun, index: int):
    return container.get(ServicesResolver).resolve(fun, index, Path(getNotebookPath()))  # pylint: disable = no-member

class pipelineFunction:

    def __init__(self, *args, **kwargs): # pylint: disable = unused-argument
        checkArgs(args, self.__class__.__name__)

    def __call__(self, fun, *args, **kwargs):
        services = _resolveServices(fun, 0)
        fun(*services)

        return fun

class dataFrameLoader:

    def __init__(self, *args, **kwargs):
        self._displayEnabled = kwargs.get('display', False)
        checkArgs(args, self.__class__.__name__)

    def __call__(self, fun, *args, **kwargs):
        services = _resolveServices(fun, 0)
        df = loadDataFrame(fun, services)

        if self._displayEnabled:
            display(df)

        return fun

class transformation:

    def __init__(self, *args, **kwargs):
        self._sources = args # type: Tuple[callable]
        self._displayEnabled = kwargs.get('display', False)

    def __call__(self, fun, *args, **kwargs):
        services = _resolveServices(fun, len(self._sources))
        df = transform(fun, self._sources, services)

        if self._displayEnabled:
            display(df)

        return fun

class dataFrameSaver:

    def __init__(self, *args):
        self._sources = args # type: Tuple[callable]

    def __call__(self, fun, *args, **kwargs):
        services = _resolveServices(fun, 1)
        saveDataFrame(fun, self._sources, services)

        return fun



from typing import List
from pyspark.sql import DataFrame

def loadDataFrame(fun: callable, services: List[object]) -> DataFrame:
    g = fun.__globals__
    df = fun(*services)
    g[fun.__name__ + '_df'] = df

    return df
